@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 1996-2013 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2013 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -27,7 +27,6 @@
 
 
 #include "conf.h"
-#include "mygetopt.h"
 #include "file.h"
 #include "packer.h"
 #include "p_elf.h"
@@ -63,7 +62,7 @@ void options_t::reset()
     o->console = CON_FILE;
 #if defined(__DJGPP__)
     o->console = CON_INIT;
-#elif defined(USE_SCREEN_WIN32)
+#elif (USE_SCREEN_WIN32)
     o->console = CON_INIT;
 #elif 1 && defined(__linux__)
     o->console = CON_INIT;
@@ -90,6 +89,19 @@ static int done_script_name = 0;
 const char *argv0 = "";
 const char *progname = "";
 
+static acc_getopt_t mfx_getopt;
+#define mfx_optarg      mfx_getopt.optarg
+#define mfx_optind      mfx_getopt.optind
+#define mfx_option      acc_getopt_longopt_t
+static void handle_opterr(acc_getopt_p g, const char *f, void *v)
+{
+    struct A { va_list ap; };
+    struct A *a = (struct A *) v;
+    fprintf( stderr, "%s: ", g->progname);
+    vfprintf(stderr, f, a->ap);
+    fprintf( stderr, "\n");
+}
+
 
 static int num_files = -1;
 static int exit_code = EXIT_OK;
@@ -883,6 +895,12 @@ static int do_option(int optc, const char *arg)
     case 673:
         opt->ps1_exe.do_8mib = false;
         break;
+    case 674:
+        opt->o_unix.unmap_all_pages = true;  // val ?
+        break;
+    case 675:
+        opt->o_unix.preserve_build_id = true;
+        break;
 
     case '\0':
         return -1;
@@ -1032,6 +1050,8 @@ static const struct mfx_option longopts[] =
     {"netbsd",           0x10, 0, 668},
     {"OpenBSD",          0x10, 0, 669},
     {"openbsd",          0x10, 0, 669},
+    {"unmap-all-pages",  0x10, 0, 674},     // linux /proc/self/exe vanishes
+    {"preserve-build-id",   0, 0, 675},
     // watcom/le
     {"le",               0x10, 0, 620},     // produce LE output
     // win32/pe
@@ -1057,13 +1077,14 @@ static const struct mfx_option longopts[] =
 };
 
     int optc, longind;
-    char buf[256];
+    char shortopts[256];
 
-    prepare_shortopts(buf,"123456789hH?V",longopts),
-    mfx_optind = 0;
-    mfx_opterr = 1;
+    prepare_shortopts(shortopts, "123456789hH?V", longopts),
+    acc_getopt_init(&mfx_getopt, 1, argc, argv);
+    mfx_getopt.progname = progname;
+    mfx_getopt.opterr = handle_opterr;
     opt->o_unix.osabi0 = Elf32_Ehdr::ELFOSABI_LINUX;
-    while ((optc = mfx_getopt_long(argc, argv, buf, longopts, &longind)) >= 0)
+    while ((optc = acc_getopt(&mfx_getopt, shortopts, longopts, &longind)) >= 0)
     {
         if (do_option(optc, argv[mfx_optind-1]) != 0)
             e_usage();
@@ -1149,7 +1170,7 @@ static const struct mfx_option longopts[] =
     int targc;
     char **targv = NULL;
     static const char sep[] = " \t";
-    char buf[256];
+    char shortopts[256];
 
     var = getenv(OPTIONS_VAR);
     if (var == NULL || !var[0])
@@ -1205,10 +1226,11 @@ static const struct mfx_option longopts[] =
             e_envopt(targv[i]);
 
     /* handle options */
-    prepare_shortopts(buf,"123456789",longopts);
-    mfx_optind = 0;
-    mfx_opterr = 1;
-    while ((optc = mfx_getopt_long(targc, targv, buf, longopts, &longind)) >= 0)
+    prepare_shortopts(shortopts, "123456789", longopts);
+    acc_getopt_init(&mfx_getopt, 1, targc, targv);
+    mfx_getopt.progname = progname;
+    mfx_getopt.opterr = handle_opterr;
+    while ((optc = acc_getopt(&mfx_getopt, shortopts, longopts, &longind)) >= 0)
     {
         if (do_option(optc, targv[mfx_optind-1]) != 0)
             e_envopt(NULL);
@@ -1267,9 +1289,13 @@ static bool test(void)
     COMPILE_TIME_ASSERT_ALIGNED1(test2_t)
     COMPILE_TIME_ASSERT(sizeof(t2) == 7 + 21*sizeof(T))
 #if defined(__acc_alignof)
+#if (ACC_CC_CLANG && ACC_ARCH_AMD64)
+    // CBUG: clang-2.8 bug in 64-bit mode
+#else
     COMPILE_TIME_ASSERT(__acc_alignof(t1) == 1)
     COMPILE_TIME_ASSERT(__acc_alignof(t2) == 1)
 #endif
+#endif
 #if 1 && (ACC_CC_WATCOMC)
     test1_t t11; COMPILE_TIME_ASSERT(sizeof(t11.a) <= sizeof(t11.b))
     test2_t t22; COMPILE_TIME_ASSERT(sizeof(t22.a) <= sizeof(t22.b))
@@ -1347,27 +1373,40 @@ void upx_sanity_check(void)
     assert(TestBELE<BE32>::test());
     assert(TestBELE<BE64>::test());
     {
-    static const unsigned char dd[32] = { 0, 0, 0, 0, 0, 0, 0,
-         0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8,
-         0, 0, 0, 0,
-         0x7f, 0x7e, 0x7d, 0x7c, 0x7b, 0x7a, 0x79, 0x78,
+    static const unsigned char dd[32]
+#if 1 && (ACC_CC_CLANG || ACC_CC_GNUC || ACC_CC_INTELC || ACC_CC_PATHSCALE) && defined(__ELF__)
+        __attribute__((__aligned__(16)))
+#endif
+        = { 0, 0, 0, 0, 0, 0, 0,
+        0xff, 0xfe, 0xfd, 0xfc, 0xfb, 0xfa, 0xf9, 0xf8,
+        0, 0, 0, 0,
+        0x7f, 0x7e, 0x7d, 0x7c, 0x7b, 0x7a, 0x79, 0x78,
     0, 0, 0, 0, 0 };
     const unsigned char *d;
+    const N_BELE_RTP::AbstractPolicy *bele;
     d = dd + 7;
     assert(upx_adler32(d, 4) == 0x09f003f7);
     assert(upx_adler32(d, 4, 0) == 0x09ec03f6);
     assert(upx_adler32(d, 4, 1) == 0x09f003f7);
+    bele = &N_BELE_RTP::be_policy;
     assert(get_be16(d) == 0xfffe);
+    assert(bele->get16(d) == 0xfffe);
     assert(get_be16_signed(d) == -2);
     assert(get_be24(d) == 0xfffefd);
+    assert(bele->get24(d) == 0xfffefd);
     assert(get_be24_signed(d) == -259);
     assert(get_be32(d) == 0xfffefdfc);
+    assert(bele->get32(d) == 0xfffefdfc);
     assert(get_be32_signed(d) == -66052);
+    bele = &N_BELE_RTP::le_policy;
     assert(get_le16(d) == 0xfeff);
+    assert(bele->get16(d) == 0xfeff);
     assert(get_le16_signed(d) == -257);
     assert(get_le24(d) == 0xfdfeff);
+    assert(bele->get24(d) == 0xfdfeff);
     assert(get_le24_signed(d) == -131329);
     assert(get_le32(d) == 0xfcfdfeff);
+    assert(bele->get32(d) == 0xfcfdfeff);
     assert(get_le32_signed(d) == -50462977);
     assert(get_le64_signed(d) == ACC_INT64_C(-506097522914230529));
     assert(find_be16(d, 2, 0xfffe) == 0);
@@ -1388,7 +1427,7 @@ void upx_sanity_check(void)
 // main entry point
 **************************************************************************/
 
-#if !defined(WITH_GUI)
+#if !(WITH_GUI)
 
 #if (ACC_ARCH_M68K && ACC_OS_TOS && ACC_CC_GNUC) && defined(__MINT__)
 extern "C" { extern long _stksize; long _stksize = 256 * 1024L; }
@@ -1403,6 +1442,9 @@ int __acc_cdecl_main main(int argc, char *argv[])
 #if 0 && defined(__DJGPP__)
     // LFN=n may cause problems with 2.03's _rename and mkdir under WinME
     putenv("LFN=y");
+#endif
+#if (ACC_OS_WIN32 || ACC_OS_WIN64) && (ACC_CC_MSC) && defined(_WRITE_ABORT_MSG) && defined(_CALL_REPORTFAULT)
+    _set_abort_behavior(_WRITE_ABORT_MSG, _WRITE_ABORT_MSG | _CALL_REPORTFAULT);
 #endif
     acc_wildargv(&argc, &argv);
 
@@ -1438,7 +1480,7 @@ int __acc_cdecl_main main(int argc, char *argv[])
 
     set_term(stderr);
 
-#if defined(WITH_UCL)
+#if (WITH_UCL)
     if (ucl_init() != UCL_E_OK)
     {
         show_head();
@@ -1449,7 +1491,7 @@ int __acc_cdecl_main main(int argc, char *argv[])
         e_exit(EXIT_INIT);
     }
 #endif
-#if defined(WITH_NRV)
+#if (WITH_NRV)
     if (nrv_init() != NRV_E_OK || NRV_VERSION != nrv_version())
     {
         show_head();
@@ -1542,7 +1584,7 @@ int __acc_cdecl_main main(int argc, char *argv[])
     return exit_code;
 }
 
-#endif /* !defined(WITH_GUI) */
+#endif /* !(WITH_GUI) */
 
 
 /*
